;----------------------
; kernel32 include
;----------------------


;-----------------------------
; Functions
;-----------------------------

from_dll 	kernel32.dll

import		CreateFile 		alias	CreateFileA
import		CloseHandle
import		CreateProcess		alias	CreateProcessA
import		CreateThread
import		CreateEvent		alias	CreateEventA
import		ContinueDebugEvent

import		ExitProcess
import		ExitThread

import		FlushInstructionCache

import		GetModuleHandle		alias	GetModuleHandleA	
import		GetStdHandle
import		GetCommandLine 		alias	GetCommandLineA
import		GetCurrentProcess
import		GetFileSize
import		GetLastError
import		GetLocalTime [1]
import		GetStartupInfo		alias	GetStartupInfoA
import		GetThreadContext

import		GlobalAlloc
import		GlobalFree
import		GlobalLock
import		GlobalUnlock
import		GetProcAddress
import		GetTickCount

import		LoadLibrary		alias LoadLibraryA
import		OutputDebugString	alias OutputDebugStringA

import		ReadFile
import		ReadProcessMemory
import		ResetEvent

import		SetFilePointer	
import		SetEvent
import		SetPriorityClass
import		Sleep
import		SetThreadContext

import		VirtualAlloc
import		VirtualFree

import		WriteFile
import		WriteProcessMemory
import		WaitForDebugEvent
import		WaitForMultipleObjects
import		WaitForSingleObject

import		QueryPerformanceFrequency
import		QueryPerformanceCounter


;---------------------
; EQU's
;---------------------

;----------------------------
; debug events
;----------------------------
EXCEPTION_DEBUG_EVENT			equ	1
CREATE_THREAD_DEBUG_EVENT		equ	2
CREATE_PROCESS_DEBUG_EVENT		equ	3
EXIT_THREAD_DEBUG_EVENT			equ	4
EXIT_PROCESS_DEBUG_EVENT		equ	5
LOAD_DLL_DEBUG_EVENT			equ	6
UNLOAD_DLL_DEBUG_EVENT			equ	7
OUTPUT_DEBUG_STRING_EVENT		equ	8
RIP_EVENT				equ	9


DBG_CONTINUE				equ	00010002h
DBG_TERMINATE_THREAD			equ	40010003h
DBG_TERMINATE_PROCESS			equ	40010004h
DBG_CONTROL_C				equ	40010005h
DBG_CONTROL_BREAK			equ	40010008h
DBG_EXCEPTION_NOT_HANDLED		equ	80010001h


;---------------------------------
; STD handles
;---------------------------------
STD_INPUT_HANDLE		equ	-10
STD_OUTPUT_HANDLE		equ	-11
STD_ERROR_HANDLE		equ	-12

;---------------------
; Thread Priority
;---------------------
NORMAL_PRIORITY_CLASS		equ	20h
IDLE_PRIORITY_CLASS		equ	40h
HIGH_PRIORITY_CLASS		equ	80h
REALTIME_PRIORITY_CLASS		equ	100h

;-------------------
; DLL main
;-------------------
DLL_PROCESS_DETACH		equ	0
DLL_PROCESS_ATTACH		equ	1
DLL_THREAD_ATTACH		equ	2
DLL_THREAD_DETACH		equ	3

WAIT_INFINITE			equ	-1


;---------------------------------
; Memory Allocation
;---------------------------------
GMEM_FIXED			EQU	0000h
GMEM_MOVEABLE			EQU	0002h
GMEM_ZEROINIT			EQU	0040h
GHND				EQU	0042h		; GMEM_MOVEABLE + GMEM_ZEROINIT
GPTR				EQU	0040h		; GMEM_FIXED + GMEM_ZEROINIT

GMEM_SHARE			equ 	2000h
GMEM_DDESHARE			equ	2000h

;--------------------
; virtual Alloc
;--------------------
PAGE_NOACCESS             equ 1
PAGE_READONLY             equ 2
PAGE_READWRITE            equ 4
PAGE_WRITECOPY            equ 8
PAGE_EXECUTE              equ 10h
PAGE_EXECUTE_READ         equ 20h
PAGE_EXECUTE_READWRITE    equ 40h
PAGE_EXECUTE_WRITECOPY    equ 80h
PAGE_GUARD                equ 100h
PAGE_NOCACHE              equ 200h
MEM_COMMIT                equ 1000h
MEM_RESERVE               equ 2000h
MEM_DECOMMIT              equ 4000h
MEM_RELEASE               equ 8000h
MEM_FREE                  equ 10000h
MEM_PRIVATE               equ 20000h
MEM_MAPPED                equ 40000h
MEM_RESET                 equ 80000h     
MEM_TOP_DOWN              equ 100000h
SEC_FILE                  equ 800000h
SEC_IMAGE                 equ 1000000h
SEC_RESERVE               equ 4000000h
SEC_COMMIT                equ 8000000h
SEC_NOCACHE               equ 10000000h
MEM_IMAGE                 equ SEC_IMAGE

;----------------------------------------------
; File I/O
;----------------------------------------------

; param dwCreationDistribution
CREATE_NEW				equ	1
CREATE_ALWAYS				equ	2
OPEN_EXISTING				equ	3
OPEN_ALWAYS				equ	4
TRUNCATE_EXISTING			equ	5

;sharemode
;	0	not shared

; file attributes
FILE_ATTRIBUTE_READONLY			equ	1h
FILE_ATTRIBUTE_HIDDEN			equ	2h
FILE_ATTRIBUTE_SYSTEM			equ	4h
FILE_ATTRIBUTE_DIRECTORY		equ	10h
FILE_ATTRIBUTE_ARCHIVE			equ	20h
FILE_ATTRIBUTE_NORMAL			equ	80h
FILE_ATTRIBUTE_TEMPORARY		equ	100h
FILE_ATTRIBUTE_COMPRESSED		equ	800h

; flags
FILE_FLAG_WRITE_THROUGH			equ	80000000h
FILE_FLAG_OVERLAPPED			equ	40000000h
FILE_FLAG_NO_BUFFERING			equ	20000000h
FILE_FLAG_RANDOM_ACCESS			equ	10000000h
FILE_FLAG_SEQUENTIAL_SCAN		equ	08000000h
FILE_FLAG_DELETE_ON_CLOSE		equ	04000000h
FILE_FLAG_BACKUP_SEMANTICS		equ	02000000h
FILE_FLAG_POSIX_SEMANTICS		equ	01000000h

; desired access
GENERIC_READ				equ	80000000h
GENERIC_WRITE				equ	40000000h

; share mode
FILE_SHARE_READ				equ	1
FILE_SHARE_WRITE			equ 2h

; SetFilePointer
FILE_BEGIN				equ	0
FILE_CURRENT				equ	1
FILE_END				equ	2

INVALID_HANDLE_VALUE			equ	-1


DEBUG_PROCESS				equ	1h
DEBUG_ONLY_THIS_PROCESS			equ	2h
CREATE_SUSPENDED			equ	4h
CREATE_SEPARATE_WOW_VDM			equ	800h
DETACHED_PROCESS			equ	8h


STATUS_BREAKPOINT			equ	080000003h
STATUS_SINGLE_STEP			equ	080000004h
STATUS_ACCESS_VIOLATION			equ	0C0000005h
STATUS_IN_PAGE_ERROR			equ	0C0000006h

EXCEPTION_ACCESS_VIOLATION		equ	STATUS_ACCESS_VIOLATION
EXCEPTION_BREAKPOINT			equ	STATUS_BREAKPOINT
EXCEPTION_SINGLE_STEP			equ	STATUS_SINGLE_STEP




SIZE_OF_80387_REGISTERS			equ	80
MAXIMUM_SUPPORTED_EXTENSION		equ	512
EXCEPTION_MAXIMUM_PARAMETERS		equ	15




;-------------------------
; Structures
;-------------------------

STRUC STARTUPINFO 
	cb			dd	?
	lpReserved		dd	?
	lpDesktop		dd	?
	lpTitle			dd	?
	dwX			dd	?
	dwY			dd	?
	dwXSize			dd	?
	dwYSize			dd	?
	dwXCountChars		dd	?
	dwYCountChars		dd	?
	dwFillAttribute		dd	?
	dwFlags			dd	?
	wShowWindow		dw	?
	cbReserved2		dw	?
	lpReserved2		dd	?
	hStdInput		dd	?
	hStdOutput		dd	?
	hStdError		dd	?
ENDS


STRUC PROCESS_INFORMATION
	hProcess		dd	?
	hThread			dd	?
	dwProcessId		dd	?
	dwThreadId		dd	?
ENDS




STRUC FLOATING_SAVE_AREA
	ControlWord		dd      ?
	StatusWord		dd      ?
	TagWord			dd      ?
	ErrorOffset		dd      ?
	ErrorSelector		dd      ?
	DataOffset		dd      ?
	DataSelector		dd      ?
	RegisterArea		rb	SIZE_OF_80387_REGISTERS
	Cr0NpxState		dd      ?
ENDS


STRUC CONTEXT
	ContextFlags		dd	?
	
	; CONTEXT_DEBUG_REGISTERS
	Dr0			dd      ?
	Dr1			dd      ?
	Dr2			dd      ?
	Dr3			dd      ?
	Dr6			dd      ?
	Dr7			dd      ?

	; CONTEXT_FLOATING_POINT
	FloatSave     		rs	FLOATING_SAVE_AREA,1

	; CONTEXT_SEGMENTS
	SegGs			dd      ?
	SegFs			dd      ?
	SegEs			dd      ?
	SegDs			dd      ?

	; CONTEXT_INTEGER
	Edi			dd      ?
	Esi			dd      ?
	Ebx			dd      ?
	Edx			dd      ?
	Ecx			dd      ?
	Eax			dd      ?
	
	; CONTEXT_CONTROL
	Ebp			dd      ?
	Eip			dd      ?
	SegCs			dd      ?
	EFlags			dd      ?
	Esp			dd      ?
	SegSs			dd      ?

	ExtendedRegisters 	rb	MAXIMUM_SUPPORTED_EXTENSION
ENDS

CONTEXT_i386			equ 00010000h
CONTEXT_i486			equ 00010000h
CONTEXT_CONTROL			equ CONTEXT_i386+00000001h
CONTEXT_INTEGER			equ CONTEXT_i386+00000002h
CONTEXT_SEGMENTS		equ CONTEXT_i386+00000004h
CONTEXT_FLOATING_POINT		equ CONTEXT_i386+00000008h
CONTEXT_DEBUG_REGISTERS		equ CONTEXT_i386+00000010h
CONTEXT_FULL			equ CONTEXT_CONTROL+CONTEXT_INTEGER+CONTEXT_SEGMENTS


STRUC CREATE_THREAD_DEBUG_INFO
	hThread			dd	?
	lpThreadLocalBase	dd	?
	lpStartAddress		dd	?
ENDS


STRUC CREATE_PROCESS_DEBUG_INFO
	hFile			dd	?
	hProcess		dd	?
	hThread			dd	?
	lpBaseOfImage		dd	?
	dwDebugInfoFileOffset	dd	?
	nDebugInfoSize		dd	?
	lpThreadLocalBase	dd	?	
	lpStartAddress		dd	?
	lpImageName		dd	?
	fUnicode		dw	?
ENDS


STRUC EXCEPTION_RECORD
	ExceptionCode		dd	?
	ExceptionFlags		dd	?
	pExceptionRecord	dd	?
	ExceptionAddress	dd	?	
	NumberParameters	dd	?
	ExceptionInformation	rd	EXCEPTION_MAXIMUM_PARAMETERS
ENDS


STRUC EXCEPTION_DEBUG_INFO
	ExceptionRecord		rs	EXCEPTION_RECORD,1
	dwFirstChance		dd	?
ENDS


STRUC OUTPUT_DEBUG_STRING_INFO
	lpDebugStringData	dd	?
	fUnicode		dw	?	
	nDebugStringiLength	dw	?
ENDS


STRUC RIP_INFO
	dwError			dd	?
	dwType			dd	?
ENDS


STRUC LOAD_DLL_DEBUG_INFO
	hFile			dd	?
	lpBaseOfDll		dd	?
	dwDebugInfoFileOffset	dd	?
	nDebugInfoSize		dd	?
	lpImageName		dd	?
	fUnicode		dw	?
ENDS


STRUC DEBUG_EVENT
	dwDebugEventCode	dd	?
	dwProcessId		dd	?
	dwThreadId		dd	?

	UNION
		Exception		rs	EXCEPTION_DEBUG_INFO,1
		CreateThread		rs	CREATE_THREAD_DEBUG_INFO,1
		CreateProcessInfo	rs	CREATE_PROCESS_DEBUG_INFO,1
		;DebugString		rs	OUTPUT_DEBUG_STRING_INFO,1
		LoadDll			rs	LOAD_DLL_DEBUG_INFO,1
		RipInfo			rs	RIP_INFO,1
	ENDU
ENDS



